import os
import heqms_pkg.machine_learning.ml_pvdm as ml
import heqms_pkg.rel_db.save_result as sr
import heqms_pkg.rel_db.to_db as tdb
import heqms_pkg.config as cf


def require():
    export = 'pip install -r requirements.txt'
    os.system(export)


def learning(part1, part2, deptNm):
    machine = ml.ml_pvdm()
    machine.learning(part1, part2, deptNm)  # 학습 (처음 1번만 사용하면 됌)


def save_result(part1, part2, deptNm):
    # 결과값 출력 및 csv 저장
    save = sr.save_result()
    save.result(part1, part2, deptNm)  # 학습한 모델로 결과값 저장


def to_db(part1, part2, deptNm):
    # DB테이블 생성 및 저장
    toDB = tdb.to_db()
    toDB.to_sql(part1, part2, deptNm)  # DB 입력


def run(part1, part2, deptNm):
    if deptNm not in cf.UnivConfig.deptNm:
        print('입력한 학과가 없습니다. 함수를 다시 호출하십시오.')
    else:
        if part1 not in cf.UnivConfig.partNm or part2 not in cf.UnivConfig.partNm:
            print('입력한 파트인자가 없습니다. 함수를 다시 호출하십시오.')
        else:
            choose = int(input('1. 패키지 설치 + 학습 + 결과 저장 + DB 입력 \n'
                               '2. 학습 + 결과 저장 + DB 입력 \n'
                               '3. 학습 + 결과 저장 \n'
                               '4. 결과 저장 + DB 입력 \n'
                               '5. 결과 저장만 \n'
                               '6. DB 입력만 \n'
                               '1, 2, 3, 4, 5, 6 중 선택하십시오. \n'
                               ))

            if choose == 1:
                require()
                print('==================')
                print('필수 패키지 설치 완료')
                print('==================')
                learning(part1, part2, deptNm)
                print('===============')
                print('머신러닝 학습 완료')
                print('===============')
                save_result(part1, part2, deptNm)
                print('================')
                print('결과 csv 저장 완료')
                print('================')
                to_db(part1, part2, deptNm)
                print('===========')
                print('DB 저장 완료')
                print('===========')

            elif choose == 2:
                learning(part1, part2, deptNm)
                print('===============')
                print('머신러닝 학습 완료')
                print('===============')
                save_result(part1, part2, deptNm)
                print('================')
                print('결과 csv 저장 완료')
                print('================')
                to_db(part1, part2, deptNm)
                print('===========')
                print('DB 저장 완료')
                print('===========')

            elif choose == 3:
                learning(part1, part2, deptNm)
                print('===============')
                print('머신러닝 학습 완료')
                print('===============')
                save_result(part1, part2, deptNm)
                print('================')
                print('결과 csv 저장 완료')
                print('================')

            elif choose == 4:
                save_result(part1, part2, deptNm)
                print('================')
                print('결과 csv 저장 완료')
                print('================')
                to_db(part1, part2, deptNm)
                print('===========')
                print('DB 저장 완료')
                print('===========')

            elif choose == 5:
                save_result(part1, part2, deptNm)
                print('================')
                print('결과 csv 저장 완료')
                print('================')

            elif choose == 6:
                to_db(part1, part2, deptNm)
                print('===========')
                print('DB 저장 완료')
                print('===========')

            else:
                print('입력한 숫자 : ' + str(choose))
                print('숫자가 다릅니다. 함수를 다시 호출하십시오.')




import pandas as pd
from sqlalchemy import create_engine
import sqlalchemy
import heqms_pkg.config as cf


class to_db:
    def __init__(self):
        self.dbconfig = cf.DBConfig_Dev()

    def to_sql(self, part1, part2, deptNm):
        con = self.dbconfig
        pr = cf.PriorConfig()
        name, mdf_part1, mdf_part2 = pr.priority(part1, part2)
        deptNo = cf.UnivConfig.deptNo[deptNm]
        finalName = str(deptNo) + '_DL_HEQM_' + name.upper() + '_RESULT.csv'
        output_path = cf.linuxPath.output_path
        path = output_path + "/" + finalName

        # 결과 csv 읽어오기
        input_data = pd.read_csv(path, index_col=0)
        input_data.index = input_data.index + 1
        input_data['RELEVANCE'] = round(input_data['RELEVANCE'], 3)

        # engine = sqlalchemy.create_engine("postgresql://user:password@host:port/database")
        url = "mysql+mysqldb://" + con.user + ":" + con.password + "@" + con.host + ":" + str(con.port) + "/" + con.db + '?charset=utf8'
        print(url) # url 확인

        engine = create_engine(url, encoding='utf-8')
        conn = engine.connect()

        # DB입력
        input_data.to_sql(name=finalName, con=engine, if_exists='replace', index=True, index_label='id',
                          dtype={

                              'id': sqlalchemy.types.INTEGER(),

                              str(mdf_part1).upper() + '_NM': sqlalchemy.VARCHAR(255),

                              str(mdf_part1).upper() + '_CD': sqlalchemy.types.VARCHAR(255),

                              str(mdf_part2).upper() + '_NM': sqlalchemy.types.VARCHAR(255),

                              str(mdf_part2).upper() + '_CD': sqlalchemy.types.VARCHAR(255),

                              'RELEVANCE': sqlalchemy.types.FLOAT(),

                              'DEPARTMENT_CD': sqlalchemy.types.VARCHAR(255)

                          })



import pandas as pd
from nltk.tokenize import RegexpTokenizer
from gensim.models.doc2vec import TaggedDocument
import os
import heqms_pkg.rel_db.from_db as fdb
import heqms_pkg.config as cf

class Preprocessing:
    def __init__(self):
        # 파싱된 교과목, 직업 정보 불러오기
        # self.file_path='C:/Users/dirty/python/Jupyter/dldoc2vec/input/job_subject.txt'
        self.fdb = fdb.MysqlController()

    # 토큰화 함수 생성
    def nltk_tokenizer(self, _wd):
        return RegexpTokenizer(r'\w+').tokenize(_wd.lower())

    # 폴더 생성
    def createFolder(self, directory):
        try:
            #if not os.path.exists(directory):
            if not os.path.isdir(directory):
                os.makedirs(directory)
        except OSError:
            print('에러: 해당 경로로 폴더를 생성할 수 없습니다 : ' + directory)

    # 전처리 과정
    def from_df(self, part1, part2, deptNm):
        # 파싱된 교과목, 직업 정보 텍스트 파일 가져오기
        ma1 = self.fdb.select_ma(part1, deptNm)
        ma2 = self.fdb.select_ma(part2, deptNm)

        ma1_token = pd.DataFrame(columns=['cd', 'token_description'])  # 통합 입력할 교과목 데이터프레임 생성
        ma2_token = pd.DataFrame(columns=['cd', 'token_description'])  # 통합 입력할 직업 데이터프레임 생성

        # description 및 task 컬럼 단어들을 토큰화하여 'token_description' 변수로 통합
        ma1_token['cd'] = ma1['code']
        ma1_token['token_description'] = ma1['ma'].apply(self.nltk_tokenizer)

        ma2_token['cd'] = ma2['code']
        ma2_token['token_description'] = ma2['ma'].apply(self.nltk_tokenizer)

        # 두 데이터 통합
        data = pd.concat([ma1_token, ma2_token], ignore_index=True)

        pr = cf.PriorConfig()
        name, mdf_part1, mdf_part2 = pr.priority(part1, part2)
        deptNo = cf.UnivConfig.deptNo[deptNm]

        input_path = cf.linuxPath.input_path
        model_path = cf.linuxPath.model_path
        output_path = cf.linuxPath.output_path

        self.createFolder(input_path)
        self.createFolder(model_path)
        self.createFolder(output_path)

        # 데이터 저장
        finalName = str(deptNo) + '_DL_HEQM_' + name.upper() + '_INPUT' + ".csv"
        data.to_csv(input_path+"/"+finalName, mode='w')

        return data

    def ma_code(self, part, deptNm):
        ma = self.fdb.select_ma(part, deptNm)
        ma_name = pd.DataFrame(columns=['code'])
        ma_name['code'] = ma['code']

        return ma_name

    def to_tag(self, part1, part2, deptNm):
        data = self.from_df(part1, part2, deptNm)

        # 태그문서로 변환
        doc_data = data[['cd', 'token_description']].values.tolist()
        tagged_data = [TaggedDocument(words=_d, tags=[uid]) for uid, _d in doc_data]

        return tagged_data



import pandas as pd
from gensim.models.doc2vec import Doc2Vec
import heqms_pkg.config as cf
import heqms_pkg.preprocess.preprocessing as pre
import heqms_pkg.rel_db.from_db as fdb


class save_result:
    def __init__(self):
        # 교과목, 직업 리스트 불러오기
        self.part = cf.UnivConfig().partNm
        self.dbconfig = cf.DBConfig_Dev()
        # self.deptNm = cf.UnivConfig().deptNm
        # self.subject = pre.Preprocessing().ma_name('subj', self.deptNm)
        # self.job = pre.Preprocessing().ma_name('job', self.deptNm)
        # self.subject_path = 'C:/Users/dirty/python/Jupyter/dldoc2vec/subject_name.txt'
        # self.subject = pd.read_csv(self.subject_path, names=['subject_name'])
        #
        # self.job_path = 'C:/Users/dirty/python/Jupyter/dldoc2vec/job_name.txt'
        # self.job = pd.read_csv(self.job_path, names=['job_name'])
        #
        # # 교과목 테이블 전처리
        # self.subject = self.subject.drop_duplicates(['subject_name'])
        # self.subject = self.subject.astype(str)
        # self.subject = self.subject.drop(self.subject.index[0])
        # self.subject = self.subject.reset_index(drop=True)
        # self.subject['subject_name'] = self.subject['subject_name'].str.lstrip()
        # self.subject['subject_name'] = self.subject['subject_name'].str.rstrip()
        #
        # # 직업 테이블 전처리
        # self.job = self.job.drop_duplicates(['job_name'])
        # self.job = self.job.astype(str)
        # self.job = self.job.drop(self.job.index[0])
        # self.job = self.job.reset_index(drop=True)
        # self.job['job_name'] = self.job['job_name'].str.lstrip()
        # self.job['job_name'] = self.job['job_name'].str.rstrip()

        # self.model = Doc2Vec.load('C:/Users/dirty/python/Jupyter/dldoc2vec/model/d2v.model')

    def result(self, part1, part2, deptNm):
        # 직무,직업,교과 순서 변경
        pr = cf.PriorConfig()
        name, mdf_part1, mdf_part2 = pr.priority(part1, part2)

        # 불러오기
        doc_list = pre.Preprocessing().from_df(mdf_part1, mdf_part2, deptNm)['token_description']
        nm_list = pre.Preprocessing().from_df(mdf_part1, mdf_part2, deptNm)['cd']

        # 학과명 -> 학과번호
        deptNum = cf.UnivConfig.deptNo[deptNm]
        model_path = cf.linuxPath.model_path
        fileName = str(deptNum) + '_' + name + '_d2v.model'

        model = Doc2Vec.load(model_path + "/" + fileName)
        model.random.seed(42)  # 값 고정

        # 직업명 리스트 불러오기
        part2_nm = pre.Preprocessing().ma_code(mdf_part2, deptNm)['code']  # 코드 불러오기

        final = pd.DataFrame(columns=[str(mdf_part1), str(mdf_part2), 'similarity'])  # 최종 데이터프레임 형식 생성

        for i in range(len(doc_list)):
            inferred_vector = model.infer_vector(doc_list[i])

            try:
                return_docs = model.docvecs.most_similar(positive=[inferred_vector], topn=1000)

            except TypeError as e:
                print(e)

                continue;

            # 결과 리스트 -> 테이블로 변형
            result_docs = pd.DataFrame(return_docs, columns=[str(mdf_part2), 'similarity'])
            result_docs[str(mdf_part1)] = nm_list[i]

            # for j in range(len(return_docs)):
            #     result_docs.loc[j, str(mdf_part1)] = part1_nm[i]
            #     result_docs.loc[j, str(mdf_part2)] = return_docs[j][0]
            #     result_docs.loc[j, 'similarity'] = return_docs[j][1]

            # 테이블 필터링으로 '직업'을 제외한 '교과목'만 출력
            final_docs = pd.merge(result_docs, part2_nm, left_on=str(mdf_part2), right_on='code', how='inner')
            final_docs = final_docs.drop(['code'], axis=1)

            final = final.append(final_docs[:30])

        # 파트별 코드-파트이름 가져오기
        cdnm = fdb.MysqlController().select_cdName
        cdnm_part1 = cdnm(mdf_part1, deptNm)
        cdnm_part2 = cdnm(mdf_part2, deptNm)

        # 코드에 맞는 파트이름 합치기
        final = pd.merge(final, cdnm_part1, left_on=str(mdf_part1), right_on=str(mdf_part1) + '_cd', how='inner')
        final = final.drop([str(mdf_part1) + '_cd'], axis=1)
        final = pd.merge(final, cdnm_part2, left_on=str(mdf_part2), right_on=str(mdf_part2) + '_cd', how='inner')
        final = final.drop([str(mdf_part2) + '_cd'], axis=1)

        # 기존 테이블 양식으로 변환
        hanbat_job_subject = pd.DataFrame(
            columns=[str(mdf_part1).upper() + '_NM', str(mdf_part1).upper() + '_CD', str(mdf_part2).upper() + '_NM',
                     str(mdf_part2).upper() + '_CD', 'RELEVANCE', 'DEPARTMENT_CD'])
        hanbat_job_subject[str(mdf_part1).upper() + "_CD"] = final[str(mdf_part1)]
        hanbat_job_subject[str(mdf_part2).upper() + "_CD"] = final[str(mdf_part2)]
        hanbat_job_subject['RELEVANCE'] = final['similarity']
        hanbat_job_subject['DEPARTMENT_CD'] = cf.UnivConfig.deptNo[deptNm]

        hanbat_job_subject[str(mdf_part1).upper() + '_NM'] = final[str(mdf_part1) + '_name']
        hanbat_job_subject[str(mdf_part2).upper() + '_NM'] = final[str(mdf_part2) + '_name']
        hanbat_job_subject.reset_index(drop=True, inplace=True)

        hanbat_job_subject.sort_values(by=[str(mdf_part1).upper() + '_NM', 'RELEVANCE'], axis=0,
                                       ascending=[True, False], inplace=True)
        hanbat_job_subject.reset_index(drop=True, inplace=True)

        # csv파일로 저장

        output_path = cf.linuxPath.output_path
        finalName = str(deptNum) + '_DL_HEQM_' + name.upper() + '_RESULT.csv'
        path = output_path + "/" + finalName
        hanbat_job_subject.to_csv(path, mode='w')





from gensim.models.doc2vec import Doc2Vec
import heqms_pkg.preprocess.preprocessing as pre
import heqms_pkg.config as cf
import multiprocessing

class ml_pvdm:
    def __init__(self):
        self.max_epochs = 2
        self.cores = multiprocessing.cpu_count()
        self.model = Doc2Vec(
            epochs=100,
            window=4,
            vector_size=1024,
            alpha=0.025,
            min_alpha=0.025,
            min_count=0,
            dm=1,
            negative=2,
            workers=self.cores,
            seed=42)

    def learning(self, part1, part2, deptNm):
        # 우선순위 변경
        name, mdf_part1, mdf_part2 = cf.PriorConfig().priority(part1, part2)

        # 불러오기
        prepro = pre.Preprocessing()
        tagged_data = prepro.to_tag(mdf_part1, mdf_part2, deptNm)
        max_epochs = self.max_epochs
        model = self.model

        model.build_vocab(tagged_data)

        # 머신러닝 학습
        for epoch in range(max_epochs):
            print('iteration {0}'.format(epoch))
            model.train(tagged_data,
                        total_examples=model.corpus_count,
                        epochs=model.iter)
            # 학습률 감소시킴
            model.alpha -= 0.002
            # 학습률 조정
            model.min_alpha = model.alpha

        deptNum = cf.UnivConfig.deptNo[deptNm] # 학과이름 -> 학과번호로 교체

        # 모델 저장
        model_path = cf.linuxPath.model_path
        fileName = str(deptNum) + '_' + name + '_d2v.model'
        model.save(model_path + "/" + fileName)
        print("Model Saved")

        return model




import setuptools

with open("heqms_pkg/README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="heqmsPkg",
    version="0.0.15",
    install_requires=[
        "bleach==3.2.1",
        "certifi==2020.12.5",
        "cffi==1.14.4",
        "chardet==4.0.0",
        "click==7.1.2",
        "colorama==0.4.4",
        "cryptography==3.3.1",
        "Cython==0.29.14",
        "docutils==0.16",
        "gensim==3.8.3",
        "idna==2.10",
        "joblib==1.0.0",
        "keyring==21.8.0",
        "mysqlclient==2.0.3",
        "nltk==3.5",
        "numpy==1.19.2",
        "packaging==20.8",
        "pandas==1.2.0",
        "pkginfo==1.6.1",
        "pycparser==2.20",
        "Pygments==2.7.3",
        "PyMySQL==1.0.1",
        "pyparsing==2.4.7",
        "pytz==2020.5",
        "regex==2020.11.13",
        "requests==2.25.1",
        "rfc3986==1.4.0",
        "scipy==1.6.0",
        "six==1.15.0",
        "SQLAlchemy==1.3.21",
        "tqdm==4.55.1",
        "urllib3==1.26.2",
        "webencodings==0.5.1",
        "wincertstore==0.2"
    ],
    author="WAF",
    author_email="iyyeo@repia.com",
    description="test package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/yiy829/Project1",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',
)



class Config:
    APP_NAME = 'myapp'
    SECRET_KEY = 'secret-key-of-myapp'
    ADMIN_NAME = 'administrator'

    AWS_DEFAULT_REGION = 'ap-northeast-2'

    STATIC_PREFIX_PATH = 'static'
    ALLOWED_IMAGE_FORMATS = ['jpg', 'jpeg', 'png', 'gif']
    MAX_IMAGE_SIZE = 5242880  # 5MB


class DBConfig_Dev(Config):
    host = '125.132.25.172'
    port = 3307
    user = 'heqms'
    password = 'heqms_00!!'
    db = 'HEQMS'


class UnivConfig(Config):
    partName = {
        '직업': 'job',
        '직무': 'au',
        '교과': 'subj',
        'job': 'job',
        'au': 'au',
        'subj': 'subj'
        }
    deptNm = ['기계공학과',
              '신소재공학과',
              '화학생명공학과',
              '산업경영공학과',
              '설비공학과',
              '창의융합학과',
              '전기공학과',
              '전자·제어공학과',
              '컴퓨터공학과',
              '정보통신공학과',
              '건설환경공학과',
              '도시공학과',
              '건축공학과',
              '건축학과',
              '시각디자인학과',
              '산업디자인학과',
              '영어영문학과',
              '중국어과',
              '일본어과',
              '공공행정학과',
              '경영회계학과',
              '경제학과'
              ]

    deptNo = {'기계공학과': '81001001',
              '신소재공학과': '81001002',
              '화학생명공학과': '81001003',
              '산업경영공학과': '81001004',
              '설비공학과': '81001005',
              '창의융합학과': '81001006',
              '전기공학과': '82002001',
              '전자·제어공학과': '82002002',
              '컴퓨터공학과': '82002003',
              '정보통신공학과': '82002004',
              '건설환경공학과': '83003001',
              '도시공학과': '83003002',
              '건축공학과': '83003003',
              '건축학과': '83003004',
              '시각디자인학과': '83003005',
              '산업디자인학과': '83003008',
              '영어영문학과': '84004001',
              '중국어과': '84004002',
              '일본어과': '84004003',
              '공공행정학과': '84004004',
              '경영회계학과': '85005001',
              '경제학과': '85005002'
              }

    partNm = ['job', 'au', 'subj']

class PriorConfig(Config):
    def __init__(self):
        pass

    def priority(self, part1, part2):
        name = str(part1) + '_' + str(part2)
        mdf_name = name
        mdf_part1 = part1
        mdf_part2 = part2

        if name == 'au_job':
            mdf_name = 'job_au'
            mdf_part1 = 'au'
            mdf_part2 = 'job'

        elif name == 'subj_job':
            mdf_name = 'job_subj'
            mdf_part1 = 'job'
            mdf_part2 = 'subj'

        elif name == 'subj_au':
            mdf_name = 'au_subj'
            mdf_part1 = 'au'
            mdf_part2 = 'subj'

        else:
            pass

        return mdf_name, mdf_part1, mdf_part2

class linuxPath(Config):
    input_path = '/opt/heqms/heqms_test/input'
    model_path = '/opt/heqms/heqms_test/model'
    output_path = '/opt/heqms/heqms_test/output'



import pymysql
import pandas as pd
import heqms_pkg.config as cf

class MysqlController:
    def __init__(self):
        self.config = cf.DBConfig_Dev()
        self.conn = pymysql.connect(host=self.config.host, port=self.config.port, user=self.config.user,
                                    password=self.config.password, db=self.config.db, charset='utf8')
        self.curs = self.conn.cursor()

    def select_subject(self, deptNm):
        sql = 'SELECT courseNm, lectGoalConcat_P, lectIntro_P, lectDescConcat_P FROM ai_subject_dictionary_cg WHERE deptNm LIKE ' + '"%' + deptNm + '%"'
        self.curs.execute(sql)
        res = self.curs.fetchall()

        subject_ma = pd.DataFrame(columns=['sub_nm', 'ma1', 'ma2', 'ma3'])
        sub_col = subject_ma.columns

        for i in range(len(sub_col)):
            for j in range(len(res)):
                subject_ma.loc[j, sub_col[i]] = str(res[j][i]).replace(" ", "_").replace("\\/", ",").replace(",", " ")

        return subject_ma

        self.conn.commit()
        self.conn.close()

    def select_job(self, deptNm):
        sql = 'SELECT job_name, parsing_job_overview, parsing_job_task FROM ontology_heqm_job_dictionary WHERE job_choice LIKE ' + '"%' + deptNm + '%"'
        self.curs.execute(sql)
        res = self.curs.fetchall()

        job_ma = pd.DataFrame(columns=['job_nm', 'ma1', 'ma2'])
        job_col = job_ma.columns

        for i in range(len(job_col)):
            for j in range(len(res)):
                job_ma.loc[j, job_col[i]] = str(res[j][i]).replace(" ", "_").replace("\\/", ",").replace(",", " ")

        return job_ma

        self.conn.commit()
        self.conn.close()

    def select_ma(self, part, deptNm):
        config = cf.UnivConfig.deptNo
        sql = 'SELECT ' + part + '_code, ' + part + '_ai_data FROM DL_HEQM_' + part.upper() +'_DICTIONARY' + ' WHERE '+ part +'_choice_deptCd LIKE ' + '"%' + config[deptNm] + '%"'
        self.curs.execute(sql)
        res = self.curs.fetchall()

        ma = pd.DataFrame(res, columns=['code', 'ma'])

        for i in range(len(ma)) :
            ma['ma'][i] = str(ma["ma"][i]).replace(" ", "_").replace("\\/", ",").replace(",", " ")

        return ma

        self.conn.commit()
        self.conn.close()


    def select_cdName(self, part, deptNm):
        config = cf.UnivConfig.deptNo
        sql = 'SELECT ' + part + '_code, ' + part + '_name FROM DL_HEQM_' + part.upper() +'_DICTIONARY' + ' WHERE '+ part +'_choice_deptCd LIKE ' + '"%' + config[deptNm] + '%"'
        self.curs.execute(sql)
        res = self.curs.fetchall()

        cd = pd.DataFrame(res, columns=[str(part)+'_cd', str(part)+'_name'])

        return cd

        self.conn.commit()
        self.conn.close()

def method():
    print("method executed")